/*
	rtdist.c	- generate a random weighted tree
				and output the lower-triangular matrix
				of leaf-node to leaf-node distances
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifndef FALSE
#define FALSE 0
#endif
#ifndef TRUE
#define TRUE (-1)
#endif
#ifndef NULL
#define NULL ((void *)0)
#endif

void* myCalloc (int size, int number)
{
	void* res;
	
	if (! (res = (void*) calloc (size, number)))
		printf ("Out of memory asking for %ld bytes.\n", size * (long) number);

	return res;
}


int main (int argc, char** argv)
{
	int nLeaf;
	int parent;
	int maxEdge = 1000;	/* maximum edge weight */
	unsigned seed = 0;	/* default random number seed */
	int N;
	int maxInt;
	int i, j, k;
	int** dist;		/* matrix of distances from each node to all others */
	int* degree;	/* degree of nodes */

	do
	{
		if (argc < 2 || argc > 5)
			break;

		if (1 != sscanf (argv[1], "%d", &nLeaf) || nLeaf < 1)
			break;

		if (argc > 2 && 1 != sscanf (argv[2], "%d", &maxEdge))
			break;

		if (argc > 3 && 1 != sscanf (argv[3], "%d", &seed))
			break;


		srand (seed);

		N = nLeaf + nLeaf - 2;	/* total number of nodes needed, counting interior */

		/* allocate storage */

		if (! (dist = myCalloc (N, sizeof (int *))))
			break;

		for (i=0; i < N; ++i)
			if (! (dist[i] = myCalloc (i+1, sizeof (int))))
				break;

		if (! (degree = myCalloc (N, sizeof (int))))
			break;

		/* add N-1 nodes to the tree - the root is implicitly there already */

		parent = 0;	/* only choice, the first time around */

		i = 1;
		maxInt = 0;		/* index of largest interior node so far */
		for (;;)
		{
			dist [i][parent] = 1 + (rand () % maxEdge);	/* distance to the parent */
			++ degree [parent];							/* mark the connection from parent */
			degree [i] = 1;								/* mark connection to parent */
			for (j=0; j < i; ++j)						/* compute distances to all other nodes */
			{
				dist [i][j] = dist [i][parent];
				if (j > parent)
					dist [i][j] += dist [j][parent];
				else
					dist [i][j] += dist [parent][j];
			}
			++i;								/* make a new node */
		
			if (i == N)
				break;
					
			if (maxInt < nLeaf - 2)
				++ maxInt;

			parent = rand () % maxInt;
			while (degree [parent] == 3)
				parent = (1 + parent) % maxInt;				/* pick an unsaturated parent for this node */
		}

		/* 
			now spit out the matrix of inter-leaf-node distances 
			(these are the nodes of degree 1)
		*/

		printf ("%d\n", nLeaf);

		k = 1;						/* counter for naming leaf nodes */
		for (i = 0; i < N; ++i)
		{
			if (degree[i] == 1)		/* if a leaf, spit out its distances */
			{
				printf ("OTU%-7d ", k++);
				for (j = 0; j <= i; ++j)
					if (degree[j] == 1)
						printf ("%7d ", dist[i][j]);
				putchar ('\n');
			}
		}
		
		return FALSE;

	} while (FALSE);

	puts	("Usage:");
	printf	("   %s n [e [s]]\n", argv[0]);
	puts	("where n, e, and s are positive integers.");
	puts	("This generates a random tree with n leaf nodes");
	puts	("and outputs the lower triangular matrix of");
	puts	("leaf-node to leaf-node distances.");
	puts	("The random number generator is seeded with value s.");
	puts	("The edge weights are random integers from 1 to e.");

	return TRUE;
}
